%  o o o o o o     
%    o o o o o                                           
%    o o o o o    VariSphear VSA                            
%    o o o o o    Variable Spherical Microphone Array  
%    o o o                                            
%                                                    
%    /// ASAR RESEARCH GROUP           
%
%    Cologne University of Applied Sciences         
%    Deutsche Telekom Laboratories, Berlin          
%    Technical University of Berlin                 
%    Westdeutscher Rundfunk, Cologne              
%    
%
%
% VariSphear DRV - Device Drivers : Version R11-0520  
% -----------------------------------------------------------------------
%
% Communication gateway to the VSA laser distance sensor unit. 
%
% Device Type: TOPCON (JENOPTIK) EM-30
% 
% /// Feel free to modify and share BUT: Always mark your changes please!
%
%
% [distance, flag] = vs_laser(COM, LASER_CMD)
%
% COM : COM Port, e.g. 'COM4' Type: String 
% 
% LASER_CMD : Sensor Command, Type: String
%
%      'Lo' - Turn Beam on - without measurement (returns 0)
%      'Lf' - Turn Beam off                      (returns 0)
%      'Dm' - Get Object distance [DEFAULT]      (returns distance in m) 
%
%
% ----------------------------------------------------------------------
% 
% distance:     Returns the distance in meters using the 'Dm' command
%               Returns 0 using 'Lo' or 'Lf' commands.
% flag:         Returns 0 if everything is fine, -1 in case of any error        
%               
% ----------------------------------------------------------------------
%
% Advice  :      - Remember to turn on the Sensor and to set it into the
%                  PC Remote state at 38400-7E1 (factory default)
%                  The VariSphear Manual delivers a description.
%                - Look up the COM-Port for your specific system ->  
%                  NPort Driver Software. The Laser is connected to 
%                  PORT #4 of the NPort 5410 Device Server.
%                - The sensor may fail on very dark surfaces. White Tape
%                  or a white sheet can help.                    
%                - Remember to turn the Sensor off after your session
%                  as it runs on battery power.     
%
% Dependencies :  VSAdeviceDriver.mexw32 (WIN32)
%                 VSAdeviceDriver.mexw64 (WIN64)
%     
%
% Author        :  bBrn - Benny Bernschtz   
% Version       :  20M5.11 
% Last Version  :  V1.0 VariSphear FFL
% Since V1.0    :  - New lowlevel device driver
%                  - WARNING: SYNTAX IS NOT COMPATIBLE TO FFL v1.0


function [distance, flag] = vs_laser(COMPORT, laserCmd)

if nargin==0
   disp('Error: No COM Port defined.');
   distance=0;
   flag=-1;
   return
   
elseif nargin==1
       laserCmd='Dm';  
end

COMPORT=upper(COMPORT);
comNumber = str2double(strrep(COMPORT,'COM',''));
distance=0;

if strcmp(laserCmd,'Dm');
   txBuffer=[58, 68, 109, 10, 13];
   retBytes=14;
elseif strcmp(laserCmd,'Lo');
   txBuffer=[58, 76, 111, 10, 13];
   retBytes=5;
elseif strcmp(laserCmd,'Lf');
   txBuffer=[58, 76, 102, 10, 13];
   retBytes=5;
   
else
  disp('Error: Command not known.');
  distance=0;
  flag=-1;
  return    
end

timeout=0;
while(1)    
    [laserError, laserResponse] = VSAdeviceDriver(comNumber, retBytes, 2, txBuffer, 3);

    if laserError==0
        if laserResponse(2)==79 && laserResponse(3)==107 % OK MSG Lo/Lf
           distance=0;
           flag=0;
           break

        elseif laserResponse(2)==69 && laserResponse(3)==114 %ERR
           distance=0;
           flag=-1;

        elseif laserResponse(2)==68 && laserResponse(3)==105 && size(laserResponse,2)==14% Dist Value delivered
               laser_distance=[laserResponse(6),laserResponse(7),laserResponse(9:11)]-48;
                if laser_distance(1)<0
                   laser_distance(1)=0;
                end
                distance = laser_distance(1)*10+laser_distance(2)+laser_distance(3)/10+laser_distance(4)/100+laser_distance(5)/1000; 
                flag=0;
                break
        else
              distance=0;
              flag=-1;        
        end
    else
        distance=0;
        flag=-1;
    end
    
    timeout=timeout+1;
    
    if timeout>10
       disp(['### Error: Failed']);
       break
    end
    disp(['### Error: RETRY #',num2str(timeout), ' of 10']);
    pause(0.5);
    
end


%

% % SEND CMD AND GET DATA
%        timeout=0;   
%        laser_distance=0;
%     
%        while(1)      
%            
%            fwrite(LASER_PORT,[':',laserCmd,10,13]);             %CR/LF
%            
%            laser_response=fread(LASER_PORT,3,'uchar');           % READ FIRST 3 BYTES      
% 
%            if laser_response(2)==69 & laser_response(3)==114     % ERROR MSG
%               laser_response=fread(LASER_PORT,5,'uchar');    
%               timeout=timeout+1;
%               pause(1);
% 
%            elseif laser_response(2)==79 & laser_response(3)==107 % OK MSG
%                   laser_response=fread(LASER_PORT,2,'uchar');  
%                   distance=0;
%            break
%            
%                 
%            elseif laser_response(2)==68 & laser_response(3)==105 % Dist VALUE
%                   laser_distance=fread(LASER_PORT,11,'uchar');   % READ REST OF THE BYTES
%                   break
%            end
%            
%            
%            % TIMEOUT            
%            if timeout>5
%               distance=-1;
%               break
%            end          
%        end     
% 
% if laser_distance~=0
%     
%    laser_distance=laser_distance';
%    laser_distance=[laser_distance(3),laser_distance(4),laser_distance(6:8)]-48;
%    if laser_distance(1)<0
%       laser_distance(1)=0;
%    end
%    distance=laser_distance(1)*10+laser_distance(2)+laser_distance(3)/10+laser_distance(4)/100+laser_distance(5)/1000;
% end


